@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -16,25 +16,22 @@
     #include <config.h>
 #endif
 
-#include <glib.h>
+#include <webkit/webkit.h>
+
+G_BEGIN_DECLS
 
 #if !GLIB_CHECK_VERSION (2, 14, 0)
     #define G_PARAM_STATIC_STRINGS \
     (G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)
 #endif
 
-#if GLIB_CHECK_VERSION (2, 16, 0)
-    #include <gio/gio.h>
-#endif
-#include <webkit/webkit.h>
+#if !GTK_CHECK_VERSION (2, 14, 0)
 
-G_BEGIN_DECLS
-
-#if !GLIB_CHECK_VERSION (2, 16, 0)
-
-gint
-g_strcmp0 (const gchar* string1,
-           const gchar* string2);
+gboolean
+gtk_show_uri                           (GdkScreen*         screen,
+                                        const gchar*       uri,
+                                        guint32            timestamp,
+                                        GError**           error);
 
 #endif
 
@@ -54,23 +51,6 @@ gtk_tool_item_set_tooltip_text         (GtkToolItem*       toolitem,
 
 #endif
 
-#ifndef WEBKIT_CHECK_VERSION
-
-gfloat
-webkit_web_view_get_zoom_level         (WebKitWebView*     web_view);
-
-void
-webkit_web_view_set_zoom_level         (WebKitWebView*     web_view,
-                                        gfloat             zoom_level);
-
-void
-webkit_web_view_zoom_in                (WebKitWebView*     web_view);
-
-void
-webkit_web_view_zoom_out               (WebKitWebView*     web_view);
-
-#endif
-
 G_END_DECLS
 
 #endif /* __COMPAT_H__ */
